/*
AccountParams.java
Copyright (C) 2010  Belledonne Communications, Grenoble, France

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

package org.linphone.core;

import androidx.annotation.Nullable;
import androidx.annotation.NonNull;
import java.lang.StackTraceElement;
import org.linphone.mediastream.Log;

/**
* Object that is used to set the different parameters of a {@link Account}. <br/>
* <br/>
*/
public interface AccountParams {
    /**
    * Get enablement status of RTCP feedback (also known as AVPF profile). <br/>
    * <br/>
    * @return the enablement mode, which can be {@link AVPFMode#Default} (use<br/>
    * LinphoneCore's mode), {@link AVPFMode#Enabled} (avpf is enabled), or {@link AVPFMode#Disabled}<br/>
    * (disabled). <br/>
    */
    public AVPFMode getAvpfMode();

    /**
    * Enable the use of RTCP feedback (also known as AVPF profile). <br/>
    * <br/>
    * @param mode the enablement mode, which can be {@link AVPFMode#Default} (use<br/>
    * LinphoneCore's mode), {@link AVPFMode#Enabled} (avpf is enabled), or {@link AVPFMode#Disabled}<br/>
    * (disabled). <br/>
    */
    public void setAvpfMode(AVPFMode mode);

    /**
    * Get the interval between regular RTCP reports when using AVPF/SAVPF. <br/>
    * <br/>
    * @return The interval in seconds. <br/>
    */
    public int getAvpfRrInterval();

    /**
    * Set the interval between regular RTCP reports when using AVPF/SAVPF. <br/>
    * <br/>
    * @param interval The interval in seconds (between 0 and 5 seconds). <br/>
    */
    public void setAvpfRrInterval(int interval);

    /**
    * Get the conference factory uri. <br/>
    * <br/>
    * @return The uri of the conference factory.   <br/>
    */
    @Nullable
    public String getConferenceFactoryUri();

    /**
    * Set the conference factory uri. <br/>
    * <br/>
    * @param uri The uri of the conference factory.   <br/>
    */
    public void setConferenceFactoryUri(@Nullable String uri);

    /**
    * Returns the contact parameters. <br/>
    * <br/>
    * @return The previously set contact parameters.   <br/>
    */
    @Nullable
    public String getContactParameters();

    /**
    * Set optional contact parameters that will be added to the contact information<br/>
    * sent in the registration. <br/>
    * <br/>
    * @param contactParams A string contaning the additional parameters in text form,<br/>
    * like "myparam=something;myparam2=something_else"  <br/>
    * The main use case for this function is provide the proxy additional information<br/>
    * regarding the user agent, like for example unique identifier or apple push id.<br/>
    * As an example, the contact address in the SIP register sent will look like<br/>
    * <sip:joe@15.128.128.93:50421>;apple-push-id=43143-DFE23F-2323-FA2232. <br/>
    */
    public void setContactParameters(@Nullable String contactParams);

    /**
    * Return the contact URI parameters. <br/>
    * <br/>
    * @return The previously set contact URI parameters.   <br/>
    */
    @Nullable
    public String getContactUriParameters();

    /**
    * Set optional contact parameters that will be added to the contact information<br/>
    * sent in the registration, inside the URI. <br/>
    * <br/>
    * @param contactUriParams A string containing the additional parameters in text<br/>
    * form, like "myparam=something;myparam2=something_else"  <br/>
    * The main use case for this function is provide the proxy additional information<br/>
    * regarding the user agent, like for example unique identifier or apple push id.<br/>
    * As an example, the contact address in the SIP register sent will look like<br/>
    * <sip:joe@15.128.128.93:50421;apple-push-id=43143-DFE23F-2323-FA2232>. <br/>
    */
    public void setContactUriParameters(@Nullable String contactUriParams);

    /**
    * Return whether or not the + should be replaced by 00. <br/>
    * <br/>
    * @return Whether liblinphone should replace "+" by "00" in dialed numbers<br/>
    * (passed to {@link Core#invite}). <br/>
    */
    public boolean getDialEscapePlusEnabled();

    /**
    * Set whether liblinphone should replace "+" by international calling prefix in<br/>
    * dialed numbers (passed to linphone_core_invite). <br/>
    * <br/>
    * @param enable true to replace + by the international prefix, false otherwise. <br/>
    */
    public void setDialEscapePlusEnabled(boolean enable);

    /**
    * Get the domain name of the given account params. <br/>
    * <br/>
    * @return The domain name of the account params.   <br/>
    */
    @Nullable
    public String getDomain();

    /**
    * Get the account params expires. <br/>
    * <br/>
    * @return The duration of registration. <br/>
    */
    public int getExpires();

    /**
    * Sets the registration expiration time in seconds. <br/>
    * <br/>
    * @param expires The expiration time to set. <br/>
    */
    public void setExpires(int expires);

    /**
    * Get the identity of the account params. <br/>
    * <br/>
    * @return The SIP identity that belongs to this account params.   <br/>
    * @deprecated 01/03/2021 Use {@link #getIdentityAddress} instead. <br/>
    */
    @Deprecated
    @Nullable
    public String getIdentity();

    /**
    * Get the identity address of the account params. <br/>
    * <br/>
    * @return The SIP identity that belongs to this account params.   <br/>
    */
    @Nullable
    public Address getIdentityAddress();

    /**
    * Sets the user identity as a SIP address. <br/>
    * <br/>
    * This identity is normally formed with display name, username and domain, such<br/>
    * as: Alice <sip:alice@example.net> The REGISTER messages will have from and to<br/>
    * set to this identity. <br/>
    * @param identity The {@link Address} of the identity to set.   <br/>
    * @return 0 if successful, -1 otherwise. <br/>
    */
    public int setIdentityAddress(@Nullable Address identity);

    /**
    * Get the idkey property of a {@link AccountParams}. <br/>
    * <br/>
    * @return The idkey string, or null.   <br/>
    */
    @Nullable
    public String getIdkey();

    /**
    * Set the idkey property on the given account params. <br/>
    * <br/>
    * This property can the be referenced by another account params 'depends_on' to<br/>
    * create a dependency relation between them.<br/>
    * @param idkey The idkey string to associate to the given {@link AccountParams}. <br/>
    *  <br/>
    */
    public void setIdkey(@Nullable String idkey);

    /**
    * Get the prefix set for this account params. <br/>
    * <br/>
    * @return The international prefix.   <br/>
    */
    @Nullable
    public String getInternationalPrefix();

    /**
    * Set an international prefix to be automatically prepended when inviting a<br/>
    * number with {@link Core#invite}; This international prefix shall usually be the<br/>
    * country code of the country where the user is living, without "+". <br/>
    * <br/>
    * @param prefix The prefix to set (withouth the +).   <br/>
    */
    public void setInternationalPrefix(@Nullable String prefix);

    /**
    * Gets whether push notifications are available or not (Android & iOS only). <br/>
    * <br/>
    * @return true if push notifications are available, false otherwise <br/>
    */
    public boolean isPushNotificationAvailable();

    /**
    * Get The policy that is used to pass through NATs/firewalls when using this<br/>
    * account params. <br/>
    * <br/>
    * If it is set to null, the default NAT policy from the core will be used<br/>
    * instead. <br/>
    * @return The {@link NatPolicy} object in use.   <br/>
    * see: {@link Core#getNatPolicy} <br/>
    */
    @Nullable
    public NatPolicy getNatPolicy();

    /**
    * Set the policy to use to pass through NATs/firewalls when using this account<br/>
    * params. <br/>
    * <br/>
    * If it is set to null, the default NAT policy from the core will be used<br/>
    * instead. <br/>
    * @param policy The {@link NatPolicy} object.   <br/>
    * see: {@link Core#setNatPolicy} <br/>
    */
    public void setNatPolicy(@Nullable NatPolicy policy);

    /**
    * Tell if the proxy is used as the only route. <br/>
    * <br/>
    * @return enable true if enabled, false otherwise. <br/>
    */
    public boolean getOutboundProxyEnabled();

    /**
    * If enabled, the proxy will be used as the only route. <br/>
    * <br/>
    * @param enable true to enable, false otherwise. <br/>
    */
    public void setOutboundProxyEnabled(boolean enable);

    /**
    * Get default privacy policy for all calls routed through this proxy. <br/>
    * <br/>
    * @return Privacy mode as LinphonePrivacyMask <br/>
    */
    public int getPrivacy();

    /**
    * Set default privacy policy for all calls routed through this proxy. <br/>
    * <br/>
    * @param privacy {@link Privacy} to configure privacy <br/>
    */
    public void setPrivacy(int privacy);

    /**
    * Tell if the PUBLISH is enabled. <br/>
    * <br/>
    * @return true if PUBLISH request is enabled for this proxy. <br/>
    */
    public boolean getPublishEnabled();

    /**
    * Indicates either or not, PUBLISH must be issued for this {@link AccountParams}. <br/>
    * <br/>
    * @param enable If true, publish will be engaged. <br/>
    */
    public void setPublishEnabled(boolean enable);

    /**
    * Get the publish expiration time in second. <br/>
    * <br/>
    * Default value is the registration expiration value. <br/>
    * @return The expire time in seconds. <br/>
    */
    public int getPublishExpires();

    /**
    * Set the publish expiration time in second. <br/>
    * <br/>
    * @param expires The expire time in seconds. <br/>
    */
    public void setPublishExpires(int expires);

    /**
    * Indicates whether to add to the contact parameters the push notification<br/>
    * information. <br/>
    * <br/>
    * For IOS, it indicates for VOIP push notification. <br/>
    * @return true if push notification informations should be added, false<br/>
    * otherwise. <br/>
    */
    public boolean getPushNotificationAllowed();

    /**
    * Indicates whether to add to the contact parameters the push notification<br/>
    * information. <br/>
    * <br/>
    * For IOS, it indicates for VOIP push notification. <br/>
    * @param allow true to allow push notification information, false otherwise. <br/>
    */
    public void setPushNotificationAllowed(boolean allow);

    /**
    * Returns the push notification configuration. <br/>
    * <br/>
    * @return The {@link PushNotificationConfig} object.   <br/>
    */
    @NonNull
    public PushNotificationConfig getPushNotificationConfig();

    /**
    * Sets the push notification configuration. <br/>
    * <br/>
    * @param config The {@link PushNotificationConfig} object.   <br/>
    */
    public void setPushNotificationConfig(@NonNull PushNotificationConfig config);

    /**
    * Get the route of the collector end-point when using quality reporting. <br/>
    * <br/>
    * This SIP address should be used on server-side to process packets directly<br/>
    * before discarding packets. Collector address should be a non existing account<br/>
    * and will not receive any messages. If null, reports will be send to the proxy<br/>
    * domain. <br/>
    * @return The SIP address of the collector end-point.   <br/>
    */
    @Nullable
    public String getQualityReportingCollector();

    /**
    * Set the route of the collector end-point when using quality reporting. <br/>
    * <br/>
    * This SIP address should be used on server-side to process packets directly<br/>
    * before discarding packets. Collector address should be a non existing account<br/>
    * and will not receive any messages. If null, reports will be send to the proxy<br/>
    * domain. <br/>
    * @param collector route of the collector end-point, if null PUBLISH will be sent<br/>
    * to the proxy domain.   <br/>
    */
    public void setQualityReportingCollector(@Nullable String collector);

    /**
    * Indicates whether quality statistics during call should be stored and sent to a<br/>
    * collector according to RFC 6035. <br/>
    * <br/>
    * @return true if quality repotring is enabled, false otherwise. <br/>
    */
    public boolean getQualityReportingEnabled();

    /**
    * Indicates whether quality statistics during call should be stored and sent to a<br/>
    * collector according to RFC 6035. <br/>
    * <br/>
    * @param enable true to store quality statistics and send them to the collector,<br/>
    * false to disable it. <br/>
    */
    public void setQualityReportingEnabled(boolean enable);

    /**
    * Get the interval between interval reports when using quality reporting. <br/>
    * <br/>
    * @return The interval in seconds, 0 means interval reports are disabled. <br/>
    */
    public int getQualityReportingInterval();

    /**
    * Set the interval between 2 interval reports sending when using quality<br/>
    * reporting. <br/>
    * <br/>
    * If call exceed interval size, an interval report will be sent to the collector.<br/>
    * On call termination, a session report will be sent for the remaining period.<br/>
    * Value must be 0 (disabled) or positive. <br/>
    * @param interval The interval in seconds, 0 means interval reports are disabled. <br/>
    */
    public void setQualityReportingInterval(int interval);

    /**
    * Get the realm of the given account params. <br/>
    * <br/>
    * @return The realm of the account params.   <br/>
    */
    @Nullable
    public String getRealm();

    /**
    * Set the realm of the given account params. <br/>
    * <br/>
    * @param realm New realm value.   <br/>
    */
    public void setRealm(@Nullable String realm);

    /**
    * Get the persistent reference key associated to the account params. <br/>
    * <br/>
    * The reference key can be for example an id to an external database. It is<br/>
    * stored in the config file, thus can survive to process exits/restarts.<br/>
    * @return The reference key string that has been associated to the account<br/>
    * params, or null if none has been associated.   <br/>
    */
    @Nullable
    public String getRefKey();

    /**
    * Associate a persistent reference key to the account params. <br/>
    * <br/>
    * The reference key can be for example an id to an external database. It is<br/>
    * stored in the config file, thus can survive to process exits/restarts.<br/>
    * @param refkey The reference key string to associate to the account params.   <br/>
    */
    public void setRefKey(@Nullable String refkey);

    /**
    * Returns whether the account params is enabled or not. <br/>
    * <br/>
    * @return true if registration to the proxy is enabled. <br/>
    */
    public boolean getRegisterEnabled();

    /**
    * Indicates either or not, REGISTRATION must be issued for this {@link AccountParams}<br/>
    * . <br/>
    * <br/>
    * @param enable If true, registration will be engaged. <br/>
    */
    public void setRegisterEnabled(boolean enable);

    /**
    * Indicates whether to add to the contact parameters the push notification<br/>
    * information. <br/>
    * <br/>
    * @return true if remote push notification informations should be added, false<br/>
    * otherwise. <br/>
    */
    public boolean getRemotePushNotificationAllowed();

    /**
    * Indicates whether to add to the contact parameters the push notification<br/>
    * information. <br/>
    * <br/>
    * @param allow true to allow remote push notification information, false<br/>
    * otherwise. <br/>
    */
    public void setRemotePushNotificationAllowed(boolean allow);

    /**
    * Gets the list of the routes set for this account params. <br/>
    * <br/>
    * warning: If linphone_account_params_is_outbound_proxy_enabled is true then it<br/>
    * will only return the proxy address. <br/>
    * @return The list of routes.    <br/>
    */
    @NonNull
    public Address[] getRoutesAddresses();

    /**
    * Sets a list of SIP route. <br/>
    * <br/>
    * When a route is set, all outgoing calls will go to the route's destination if<br/>
    * this account is the default one (see {@link Core#setDefaultAccount}).<br/>
    * warning: This function cannot be used if<br/>
    * linphone_account_params_is_outbound_proxy_enabled is true. <br/>
    * @param routes A list of routes.    <br/>
    * @return -1 if routes are invalid, 0 otherwise. <br/>
    */
    public int setRoutesAddresses(@Nullable Address[] routes);

    /**
    * Get the account params proxy address. <br/>
    * <br/>
    * @return The proxy's SIP address.   <br/>
    * @deprecated 01/03/2021 Use {@link #getServerAddress} instead. <br/>
    */
    @Deprecated
    @Nullable
    public String getServerAddr();

    /**
    * Sets the proxy address. <br/>
    * <br/>
    * Examples of valid sip proxy address are:<br/>
    * @param serverAddress The proxy address to set.   <br/>
    * @return 0 if successful, -1 otherwise. <br/>
    * @deprecated 01/03/2021 Use {@link #setServerAddress} instead. <br/>
    */
    @Deprecated
    public int setServerAddr(@Nullable String serverAddress);

    /**
    * Get the account params proxy address. <br/>
    * <br/>
    * @return The proxy's SIP {@link Address}.   <br/>
    */
    @Nullable
    public Address getServerAddress();

    /**
    * Sets the proxy address. <br/>
    * <br/>
    * Examples of valid sip proxy address are:<br/>
    * @param serverAddress The proxy as {@link Address} to set.   <br/>
    * @return 0 if successful, -1 otherwise. <br/>
    */
    public int setServerAddress(@Nullable Address serverAddress);

    /**
    * Returns the transport type of the server address. <br/>
    * <br/>
    * @return The {@link TransportType} of the server address. <br/>
    */
    public TransportType getTransport();

    /**
    * Sets the transport type of the server address. <br/>
    * <br/>
    * @param transport The {@link TransportType} to set. <br/>
    */
    public void setTransport(TransportType transport);

    /**
    * Return whether or not the international prefix will automaticaly be used for<br/>
    * calls and chats. <br/>
    * <br/>
    * @return Whether we should use international prefix automatically for calls. <br/>
    */
    public boolean getUseInternationalPrefixForCallsAndChats();

    /**
    * If enabled, the international prefix will automaticaly be used for calls and<br/>
    * chats. <br/>
    * <br/>
    * @param enable true to use the international prefix for calls, false otherwise. <br/>
    */
    public void setUseInternationalPrefixForCallsAndChats(boolean enable);

    /**
    * Instantiate a new account params with values from source. <br/>
    * <br/>
    * @return The newly created {@link AccountParams} object.   <br/>
    */
    @NonNull
    public AccountParams clone();

    /**
    * Create a new {@link AccountParams} object from a configuration. <br/>
    * <br/>
    * @param lc The {@link Core} object.   <br/>
    * @param index The index of the configuration. <br/>
    * @return The newly created {@link AccountParams} object.   <br/>
    */
    @NonNull
    public AccountParams newWithConfig(@NonNull Core lc, int index);

    /**
      * Sets the object to store in this object user's data
      * @param data the object to store
      */
    public void setUserData(Object data);

    /**
      * Gets the object stored in this object user's data
      * @return the object store if any, null otherwise
      */
    public Object getUserData();

    /**
      * Gets the native pointer used by this class to make native method calls.
      * @return the nativer pointer, as long
      */
    public long getNativePointer();

    public String toString();
}

class AccountParamsImpl implements AccountParams {

    protected long nativePtr = 0;
    protected transient Object userData = null;
    protected boolean _isConst = false;

    protected AccountParamsImpl(long ptr, boolean isConst) {
        nativePtr = ptr;
        _isConst = isConst;
    }

    public long getNativePointer() {
        return nativePtr;
    }


    public boolean isConst() {
        return _isConst;
    }

    private native int getAvpfMode(long nativePtr);
    @Override
    synchronized public AVPFMode getAvpfMode()  {
        
        
        return AVPFMode.fromInt(getAvpfMode(nativePtr));
    }

    private native void setAvpfMode(long nativePtr, int mode);
    @Override
    synchronized public void setAvpfMode(AVPFMode mode)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setAvpfMode() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setAvpfMode(nativePtr, mode.toInt());
    }

    private native int getAvpfRrInterval(long nativePtr);
    @Override
    synchronized public int getAvpfRrInterval()  {
        
        
        return getAvpfRrInterval(nativePtr);
    }

    private native void setAvpfRrInterval(long nativePtr, int interval);
    @Override
    synchronized public void setAvpfRrInterval(int interval)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setAvpfRrInterval() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setAvpfRrInterval(nativePtr, interval);
    }

    private native String getConferenceFactoryUri(long nativePtr);
    @Override @Nullable
    synchronized public String getConferenceFactoryUri()  {
        
        
        return getConferenceFactoryUri(nativePtr);
    }

    private native void setConferenceFactoryUri(long nativePtr, String uri);
    @Override
    synchronized public void setConferenceFactoryUri(@Nullable String uri)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setConferenceFactoryUri() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setConferenceFactoryUri(nativePtr, uri);
    }

    private native String getContactParameters(long nativePtr);
    @Override @Nullable
    synchronized public String getContactParameters()  {
        
        
        return getContactParameters(nativePtr);
    }

    private native void setContactParameters(long nativePtr, String contactParams);
    @Override
    synchronized public void setContactParameters(@Nullable String contactParams)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setContactParameters() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setContactParameters(nativePtr, contactParams);
    }

    private native String getContactUriParameters(long nativePtr);
    @Override @Nullable
    synchronized public String getContactUriParameters()  {
        
        
        return getContactUriParameters(nativePtr);
    }

    private native void setContactUriParameters(long nativePtr, String contactUriParams);
    @Override
    synchronized public void setContactUriParameters(@Nullable String contactUriParams)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setContactUriParameters() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setContactUriParameters(nativePtr, contactUriParams);
    }

    private native boolean getDialEscapePlusEnabled(long nativePtr);
    @Override
    synchronized public boolean getDialEscapePlusEnabled()  {
        
        
        return getDialEscapePlusEnabled(nativePtr);
    }

    private native void setDialEscapePlusEnabled(long nativePtr, boolean enable);
    @Override
    synchronized public void setDialEscapePlusEnabled(boolean enable)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setDialEscapePlusEnabled() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setDialEscapePlusEnabled(nativePtr, enable);
    }

    private native String getDomain(long nativePtr);
    @Override @Nullable
    synchronized public String getDomain()  {
        
        
        return getDomain(nativePtr);
    }

    private native int getExpires(long nativePtr);
    @Override
    synchronized public int getExpires()  {
        
        
        return getExpires(nativePtr);
    }

    private native void setExpires(long nativePtr, int expires);
    @Override
    synchronized public void setExpires(int expires)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setExpires() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setExpires(nativePtr, expires);
    }

    private native String getIdentity(long nativePtr);
    @Override @Nullable
    synchronized public String getIdentity()  {
        
        
        return getIdentity(nativePtr);
    }

    private native Address getIdentityAddress(long nativePtr);
    @Override @Nullable
    synchronized public Address getIdentityAddress()  {
        
        
        return (Address)getIdentityAddress(nativePtr);
    }

    private native int setIdentityAddress(long nativePtr, Address identity);
    @Override
    synchronized public int setIdentityAddress(@Nullable Address identity)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setIdentityAddress() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        return setIdentityAddress(nativePtr, identity);
    }

    private native String getIdkey(long nativePtr);
    @Override @Nullable
    synchronized public String getIdkey()  {
        
        
        return getIdkey(nativePtr);
    }

    private native void setIdkey(long nativePtr, String idkey);
    @Override
    synchronized public void setIdkey(@Nullable String idkey)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setIdkey() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setIdkey(nativePtr, idkey);
    }

    private native String getInternationalPrefix(long nativePtr);
    @Override @Nullable
    synchronized public String getInternationalPrefix()  {
        
        
        return getInternationalPrefix(nativePtr);
    }

    private native void setInternationalPrefix(long nativePtr, String prefix);
    @Override
    synchronized public void setInternationalPrefix(@Nullable String prefix)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setInternationalPrefix() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setInternationalPrefix(nativePtr, prefix);
    }

    private native boolean isPushNotificationAvailable(long nativePtr);
    @Override
    synchronized public boolean isPushNotificationAvailable()  {
        
        
        return isPushNotificationAvailable(nativePtr);
    }

    private native NatPolicy getNatPolicy(long nativePtr);
    @Override @Nullable
    synchronized public NatPolicy getNatPolicy()  {
        
        
        return (NatPolicy)getNatPolicy(nativePtr);
    }

    private native void setNatPolicy(long nativePtr, NatPolicy policy);
    @Override
    synchronized public void setNatPolicy(@Nullable NatPolicy policy)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setNatPolicy() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setNatPolicy(nativePtr, policy);
    }

    private native boolean getOutboundProxyEnabled(long nativePtr);
    @Override
    synchronized public boolean getOutboundProxyEnabled()  {
        
        
        return getOutboundProxyEnabled(nativePtr);
    }

    private native void setOutboundProxyEnabled(long nativePtr, boolean enable);
    @Override
    synchronized public void setOutboundProxyEnabled(boolean enable)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setOutboundProxyEnabled() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setOutboundProxyEnabled(nativePtr, enable);
    }

    private native int getPrivacy(long nativePtr);
    @Override
    synchronized public int getPrivacy()  {
        
        
        return getPrivacy(nativePtr);
    }

    private native void setPrivacy(long nativePtr, int privacy);
    @Override
    synchronized public void setPrivacy(int privacy)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setPrivacy() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setPrivacy(nativePtr, privacy);
    }

    private native boolean getPublishEnabled(long nativePtr);
    @Override
    synchronized public boolean getPublishEnabled()  {
        
        
        return getPublishEnabled(nativePtr);
    }

    private native void setPublishEnabled(long nativePtr, boolean enable);
    @Override
    synchronized public void setPublishEnabled(boolean enable)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setPublishEnabled() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setPublishEnabled(nativePtr, enable);
    }

    private native int getPublishExpires(long nativePtr);
    @Override
    synchronized public int getPublishExpires()  {
        
        
        return getPublishExpires(nativePtr);
    }

    private native void setPublishExpires(long nativePtr, int expires);
    @Override
    synchronized public void setPublishExpires(int expires)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setPublishExpires() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setPublishExpires(nativePtr, expires);
    }

    private native boolean getPushNotificationAllowed(long nativePtr);
    @Override
    synchronized public boolean getPushNotificationAllowed()  {
        
        
        return getPushNotificationAllowed(nativePtr);
    }

    private native void setPushNotificationAllowed(long nativePtr, boolean allow);
    @Override
    synchronized public void setPushNotificationAllowed(boolean allow)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setPushNotificationAllowed() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setPushNotificationAllowed(nativePtr, allow);
    }

    private native PushNotificationConfig getPushNotificationConfig(long nativePtr);
    @Override @NonNull
    synchronized public PushNotificationConfig getPushNotificationConfig()  {
        
        
        return (PushNotificationConfig)getPushNotificationConfig(nativePtr);
    }

    private native void setPushNotificationConfig(long nativePtr, PushNotificationConfig config);
    @Override
    synchronized public void setPushNotificationConfig(@NonNull PushNotificationConfig config)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setPushNotificationConfig() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setPushNotificationConfig(nativePtr, config);
    }

    private native String getQualityReportingCollector(long nativePtr);
    @Override @Nullable
    synchronized public String getQualityReportingCollector()  {
        
        
        return getQualityReportingCollector(nativePtr);
    }

    private native void setQualityReportingCollector(long nativePtr, String collector);
    @Override
    synchronized public void setQualityReportingCollector(@Nullable String collector)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setQualityReportingCollector() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setQualityReportingCollector(nativePtr, collector);
    }

    private native boolean getQualityReportingEnabled(long nativePtr);
    @Override
    synchronized public boolean getQualityReportingEnabled()  {
        
        
        return getQualityReportingEnabled(nativePtr);
    }

    private native void setQualityReportingEnabled(long nativePtr, boolean enable);
    @Override
    synchronized public void setQualityReportingEnabled(boolean enable)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setQualityReportingEnabled() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setQualityReportingEnabled(nativePtr, enable);
    }

    private native int getQualityReportingInterval(long nativePtr);
    @Override
    synchronized public int getQualityReportingInterval()  {
        
        
        return getQualityReportingInterval(nativePtr);
    }

    private native void setQualityReportingInterval(long nativePtr, int interval);
    @Override
    synchronized public void setQualityReportingInterval(int interval)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setQualityReportingInterval() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setQualityReportingInterval(nativePtr, interval);
    }

    private native String getRealm(long nativePtr);
    @Override @Nullable
    synchronized public String getRealm()  {
        
        
        return getRealm(nativePtr);
    }

    private native void setRealm(long nativePtr, String realm);
    @Override
    synchronized public void setRealm(@Nullable String realm)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setRealm() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setRealm(nativePtr, realm);
    }

    private native String getRefKey(long nativePtr);
    @Override @Nullable
    synchronized public String getRefKey()  {
        
        
        return getRefKey(nativePtr);
    }

    private native void setRefKey(long nativePtr, String refkey);
    @Override
    synchronized public void setRefKey(@Nullable String refkey)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setRefKey() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setRefKey(nativePtr, refkey);
    }

    private native boolean getRegisterEnabled(long nativePtr);
    @Override
    synchronized public boolean getRegisterEnabled()  {
        
        
        return getRegisterEnabled(nativePtr);
    }

    private native void setRegisterEnabled(long nativePtr, boolean enable);
    @Override
    synchronized public void setRegisterEnabled(boolean enable)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setRegisterEnabled() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setRegisterEnabled(nativePtr, enable);
    }

    private native boolean getRemotePushNotificationAllowed(long nativePtr);
    @Override
    synchronized public boolean getRemotePushNotificationAllowed()  {
        
        
        return getRemotePushNotificationAllowed(nativePtr);
    }

    private native void setRemotePushNotificationAllowed(long nativePtr, boolean allow);
    @Override
    synchronized public void setRemotePushNotificationAllowed(boolean allow)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setRemotePushNotificationAllowed() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setRemotePushNotificationAllowed(nativePtr, allow);
    }

    private native Address[] getRoutesAddresses(long nativePtr);
    @Override @NonNull
    synchronized public Address[] getRoutesAddresses()  {
        
        
        return getRoutesAddresses(nativePtr);
    }

    private native int setRoutesAddresses(long nativePtr, Address[] routes);
    @Override
    synchronized public int setRoutesAddresses(@Nullable Address[] routes)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setRoutesAddresses() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        return setRoutesAddresses(nativePtr, routes);
    }

    private native String getServerAddr(long nativePtr);
    @Override @Nullable
    synchronized public String getServerAddr()  {
        
        
        return getServerAddr(nativePtr);
    }

    private native int setServerAddr(long nativePtr, String serverAddress);
    @Override
    synchronized public int setServerAddr(@Nullable String serverAddress)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setServerAddr() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        return setServerAddr(nativePtr, serverAddress);
    }

    private native Address getServerAddress(long nativePtr);
    @Override @Nullable
    synchronized public Address getServerAddress()  {
        
        
        return (Address)getServerAddress(nativePtr);
    }

    private native int setServerAddress(long nativePtr, Address serverAddress);
    @Override
    synchronized public int setServerAddress(@Nullable Address serverAddress)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setServerAddress() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        return setServerAddress(nativePtr, serverAddress);
    }

    private native int getTransport(long nativePtr);
    @Override
    synchronized public TransportType getTransport()  {
        
        
        return TransportType.fromInt(getTransport(nativePtr));
    }

    private native void setTransport(long nativePtr, int transport);
    @Override
    synchronized public void setTransport(TransportType transport)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setTransport() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setTransport(nativePtr, transport.toInt());
    }

    private native boolean getUseInternationalPrefixForCallsAndChats(long nativePtr);
    @Override
    synchronized public boolean getUseInternationalPrefixForCallsAndChats()  {
        
        
        return getUseInternationalPrefixForCallsAndChats(nativePtr);
    }

    private native void setUseInternationalPrefixForCallsAndChats(long nativePtr, boolean enable);
    @Override
    synchronized public void setUseInternationalPrefixForCallsAndChats(boolean enable)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setUseInternationalPrefixForCallsAndChats() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setUseInternationalPrefixForCallsAndChats(nativePtr, enable);
    }

    private native AccountParams clone(long nativePtr);
    @Override @NonNull
    synchronized public AccountParams clone()  {
        
        
        return (AccountParams)clone(nativePtr);
    }

    private native AccountParams newWithConfig(long nativePtr, Core lc, int index);
    @Override @NonNull
    synchronized public AccountParams newWithConfig(@NonNull Core lc, int index)  {
        
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call newWithConfig() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        return (AccountParams)newWithConfig(nativePtr, lc, index);
    }

    private native boolean unref(long ptr, boolean isConst);
    protected void finalize() throws Throwable {
        /* Considering the following scenario:
        User scrolls fast in a chat message list that contains a lot of images, 
        so the VM will garbage collect often due to the images taking much space.
        The view itself only has a reference on the EventLogs, not the ChatMessages directly,
        so each call of onBindView will call eventLog.getChatMessage() and the reference will only be temporary.
        It has been observed a race condition in which the JNI level can obtain a LocalRef from a WeakGlobalRef
        of an object that has been scheduled for destruction and for which the finalize() method will be called shortly after.
        The result is a Java object that has been finalized can have it's methods called after the finalize() has terminated.*/
        
        /*The workaround we made here is to keep the nativePtr value even after the unref() only if the object is still reffed by the SDK,
        so the few methods calls following the finalize() will still work because we know the native pointer still exists.*/

        /*This solution will work as long as the objects still have a ref at C level, which is the case with ChatMessages because they are hold by the Events.
        Should we have this problem again in a situation where the C no longer holds a reference (resulting in the C object being destroyed by finalize()), 
        another solution is possible (but more complex to implement).
        This solution would consist in implementing at C level, a queue of belle_sip_object_t that are loosing their last ref. 
        Instead of being deleted immediately by belle_sip_object_unref() calling belle_sip_object_delete(), 
        unref() would place the object onto this "C finalizer queue", the delete() call would then be performed later, 
        by a hook added into the main android looper.
        This solution should work because the actual destruction will be performed from the looper context, 
        so in a place where the local java object causing our troubles no longer exists for sure.
        This solution has however the inconvenience that it can work only within an application that uses a single thread to call liblinphone API.*/
        
        if (nativePtr != 0) {
            boolean destroyed = unref(nativePtr, _isConst);
            if (destroyed) {
                nativePtr = 0;
            }
        }
        super.finalize();
    }

    @Override
    public void setUserData(Object data) {
        userData = data;
    }

    @Override
    public Object getUserData() {
        return userData;
    }

    @Override
    public String toString() {
        return "Java object [" + super.toString() + "], native pointer [" + String.format("0x%08x", nativePtr) + "]";
    }
}
