/*
Friend.java
Copyright (C) 2010  Belledonne Communications, Grenoble, France

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

package org.linphone.core;

import androidx.annotation.Nullable;
import androidx.annotation.NonNull;
import java.lang.StackTraceElement;
import org.linphone.mediastream.Log;

/**
* This object is used to store a SIP address. <br/>
* <br/>
* {@link Friend} is mainly used to implement an adressbook feature, and are used<br/>
* as data for the {@link MagicSearch} object. If your proxy supports it, you can<br/>
* also use it to subscribe to presence information.<br/>
* The objects are stored in a {@link FriendList} which are in turn stored inside<br/>
* the {@link Core}. They can be stored inside a database if the path to it is<br/>
* configured, otherwise they will be lost after the {@link Core} is destroyed.<br/>
* Thanks to the vCard plugin, you can also store more information like phone<br/>
* numbers, organization, etc... <br/>
*/
public interface Friend {
    /**
    * Get address of this friend. <br/>
    * <br/>
    * note: the {@link Address} object returned is hold by the LinphoneFriend,<br/>
    * however calling several time this function may return different objects. <br/>
    * @return the {@link Address}.   <br/>
    */
    @Nullable
    public Address getAddress();

    /**
    * Set {@link Address} for this friend. <br/>
    * <br/>
    * @param address the {@link Address} to set   return 0 if successfull, -1<br/>
    * otherwise <br/>
    */
    public int setAddress(@Nullable Address address);

    /**
    * Returns a list of {@link Address} for this friend. <br/>
    * <br/>
    * @return A list of {@link Address}.    <br/>
    */
    @NonNull
    public Address[] getAddresses();

    /**
    * Returns the capabilities associated to this friend. <br/>
    * <br/>
    * @return an int representing the capabilities of the friend <br/>
    */
    public int getCapabilities();

    /**
    * Get the consolidated presence of a friend. <br/>
    * <br/>
    * @return The {@link ConsolidatedPresence} of the friend <br/>
    */
    public ConsolidatedPresence getConsolidatedPresence();

    /**
    * Returns the {@link Core} object managing this friend, if any. <br/>
    * <br/>
    * @return the {@link Core} object associated.   <br/>
    */
    @NonNull
    public Core getCore();

    /**
    * get current subscription policy for this {@link Friend} <br/>
    * <br/>
    * @return the {@link SubscribePolicy} enum <br/>
    */
    public SubscribePolicy getIncSubscribePolicy();

    /**
    * Configure incoming subscription policy for this friend. <br/>
    * <br/>
    * @param policy {@link SubscribePolicy} policy to apply. <br/>
    * @return 0 <br/>
    */
    public int setIncSubscribePolicy(SubscribePolicy policy);

    /**
    * Tells whether we already received presence information for a friend. <br/>
    * <br/>
    * @return true if presence information has been received for the friend, false<br/>
    * otherwise. <br/>
    */
    public boolean isPresenceReceived();

    /**
    * Get the display name for this friend. <br/>
    * <br/>
    * @return The display name of this friend.   <br/>
    */
    @Nullable
    public String getName();

    /**
    * Set the display name for this friend. <br/>
    * <br/>
    * @param name the display name to set   <br/>
    * @return 0 if successful, -1 otherwise <br/>
    */
    public int setName(@Nullable String name);

    /**
    * Returns a list of phone numbers for this friend. <br/>
    * <br/>
    * @return A list of phone numbers as string.      <br/>
    */
    @NonNull
    public String[] getPhoneNumbers();

    /**
    * Get the presence model of a friend. <br/>
    * <br/>
    * @return A {@link PresenceModel} object, or null if the friend do not have<br/>
    * presence information (in which case he is considered offline).   <br/>
    */
    @Nullable
    public PresenceModel getPresenceModel();

    /**
    * Set the presence model of a friend. <br/>
    * <br/>
    * @param presence The {@link PresenceModel} object to set for the friend   <br/>
    */
    public void setPresenceModel(@Nullable PresenceModel presence);

    /**
    * Get the reference key of a friend. <br/>
    * <br/>
    * @return The reference key of the friend.   <br/>
    */
    @Nullable
    public String getRefKey();

    /**
    * Set the reference key of a friend. <br/>
    * <br/>
    * @param key The reference key to use for the friend.   <br/>
    */
    public void setRefKey(@Nullable String key);

    /**
    * get subscription flag value <br/>
    * <br/>
    * @return returns true is subscription is activated for this friend <br/>
    */
    public boolean subscribesEnabled();

    /**
    * Configure {@link Friend} to subscribe to presence information. <br/>
    * <br/>
    * @param enable if true this friend will receive subscription message <br/>
    * @return 0 <br/>
    */
    public int enableSubscribes(boolean enable);

    /**
    * Get subscription state of a friend. <br/>
    * <br/>
    * @return the {@link SubscriptionState} enum <br/>
    */
    public SubscriptionState getSubscriptionState();

    /**
    * Returns the vCard object associated to this friend, if any. <br/>
    * <br/>
    * @return the {@link Vcard} or null.   <br/>
    */
    @Nullable
    public Vcard getVcard();

    /**
    * Binds a vCard object to a friend. <br/>
    * <br/>
    * @param vcard The {@link Vcard} object to bind   <br/>
    */
    public void setVcard(@Nullable Vcard vcard);

    /**
    * Adds an address in this friend. <br/>
    * <br/>
    * @param address {@link Address} object   <br/>
    */
    public void addAddress(@NonNull Address address);

    /**
    * Adds a phone number in this friend. <br/>
    * <br/>
    * @param phoneNumber number to add   <br/>
    */
    public void addPhoneNumber(@NonNull String phoneNumber);

    /**
    * Creates a vCard object associated to this friend if there isn't one yet and if<br/>
    * the full name is available, either by the parameter or the one in the friend's<br/>
    * SIP URI. <br/>
    * <br/>
    * @param name The full name of the friend or null to use the one from the<br/>
    * friend's SIP URI   <br/>
    * @return true if the vCard has been created, false if it wasn't possible (for<br/>
    * exemple if name and the friend's SIP URI are null or if the friend's SIP URI<br/>
    * doesn't have a display name), or if there is already one vcard <br/>
    */
    public boolean createVcard(@Nullable String name);

    /**
    * Commits modification made to the friend configuration. <br/>
    * <br/>
    */
    public void done();

    /**
    * Starts editing a friend configuration. <br/>
    * <br/>
    * Because friend configuration must be consistent, applications MUST call {@link #edit}<br/>
    * before doing any attempts to modify friend configuration (such as {@link #setAddress}<br/>
    * or {@link #setIncSubscribePolicy}). Once the modifications are done, then the<br/>
    * application must call {@link #done} to commit the changes. <br/>
    */
    public void edit();

    /**
    * Returns the version of a friend's capbility. <br/>
    * <br/>
    * @param capability {@link FriendCapability} object <br/>
    * @return the version of a friend's capbility. <br/>
    */
    public float getCapabilityVersion(FriendCapability capability);

    /**
    * Get the presence model for a specific SIP URI or phone number of a friend. <br/>
    * <br/>
    * @param uriOrTel The SIP URI or phone number for which to get the presence model<br/>
    *   <br/>
    * @return A {@link PresenceModel} object, or null if the friend do not have<br/>
    * presence information for this SIP URI or phone number.   <br/>
    */
    @Nullable
    public PresenceModel getPresenceModelForUriOrTel(@NonNull String uriOrTel);

    /**
    * Returns whether or not a friend has a capbility. <br/>
    * <br/>
    * @param capability {@link FriendCapability} object <br/>
    * @return whether or not a friend has a capbility <br/>
    */
    public boolean hasCapability(FriendCapability capability);

    /**
    * Returns whether or not a friend has a capbility with a given version. <br/>
    * <br/>
    * @param capability {@link FriendCapability} object <br/>
    * @param version the version to test <br/>
    * @return whether or not a friend has a capbility with a given version or -1.0 if<br/>
    * friend has not capability. <br/>
    */
    public boolean hasCapabilityWithVersion(FriendCapability capability, float version);

    /**
    * Returns whether or not a friend has a capbility with a given version or more. <br/>
    * <br/>
    * @param capability {@link FriendCapability} object <br/>
    * @param version the version to test <br/>
    * @return whether or not a friend has a capbility with a given version or more. <br/>
    */
    public boolean hasCapabilityWithVersionOrMore(FriendCapability capability, float version);

    /**
    * Returns whether a friend contains the given phone number. <br/>
    * <br/>
    * @param phoneNumber the phone number to search for   <br/>
    * @return true if found, false otherwise <br/>
    */
    public boolean hasPhoneNumber(@NonNull String phoneNumber);

    /**
    * Check that the given friend is in a friend list. <br/>
    * <br/>
    * @return true if the friend is in a friend list, false otherwise. <br/>
    */
    public boolean inList();

    /**
    * Removes a friend from it's friend list and from the rc if exists. <br/>
    * <br/>
    */
    public void remove();

    /**
    * Removes an address in this friend. <br/>
    * <br/>
    * @param address {@link Address} object   <br/>
    */
    public void removeAddress(@NonNull Address address);

    /**
    * Removes a phone number in this friend. <br/>
    * <br/>
    * @param phoneNumber number to remove   <br/>
    */
    public void removePhoneNumber(@NonNull String phoneNumber);

    /**
    * Saves a friend either in database if configured, otherwise in linphonerc. <br/>
    * <br/>
    * @param core the linphone core   <br/>
    */
    public void save(@NonNull Core core);

    /**
    * Set the presence model for a specific SIP URI or phone number of a friend. <br/>
    * <br/>
    * @param uriOrTel The SIP URI or phone number for which to set the presence model<br/>
    *   <br/>
    * @param presence The {@link PresenceModel} object to set   <br/>
    */
    public void setPresenceModelForUriOrTel(@NonNull String uriOrTel, @Nullable PresenceModel presence);

    /**
    * Contructor same as linphone_friend_new + {@link #setAddress} <br/>
    * <br/>
    * @param vcard a {@link Vcard} object   <br/>
    * @return a new {@link Friend} which has its vCard attribute initialized from the<br/>
    * given vCard. This can be get by {@link #getVcard}.   <br/>
    */
    @Nullable
    public Friend newFromVcard(@NonNull Vcard vcard);

    /**
      * Sets the object to store in this object user's data
      * @param data the object to store
      */
    public void setUserData(Object data);

    /**
      * Gets the object stored in this object user's data
      * @return the object store if any, null otherwise
      */
    public Object getUserData();

    /**
      * Gets the native pointer used by this class to make native method calls.
      * @return the nativer pointer, as long
      */
    public long getNativePointer();

    public String toString();
}

class FriendImpl implements Friend {

    protected long nativePtr = 0;
    protected transient Object userData = null;
    protected Core core = null;
    protected boolean _isConst = false;

    protected FriendImpl(long ptr, boolean isConst) {
        nativePtr = ptr;
        core = getCore();
        _isConst = isConst;
    }

    public long getNativePointer() {
        return nativePtr;
    }


    public boolean isConst() {
        return _isConst;
    }

    private native Address getAddress(long nativePtr);
    @Override @Nullable
    synchronized public Address getAddress()  {
        synchronized(core) { 
        
        return (Address)getAddress(nativePtr);
        }
    }

    private native int setAddress(long nativePtr, Address address);
    @Override
    synchronized public int setAddress(@Nullable Address address)  {
        synchronized(core) { 
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setAddress() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        return setAddress(nativePtr, address);
        }
    }

    private native Address[] getAddresses(long nativePtr);
    @Override @NonNull
    synchronized public Address[] getAddresses()  {
        synchronized(core) { 
        
        return getAddresses(nativePtr);
        }
    }

    private native int getCapabilities(long nativePtr);
    @Override
    synchronized public int getCapabilities()  {
        synchronized(core) { 
        
        return getCapabilities(nativePtr);
        }
    }

    private native int getConsolidatedPresence(long nativePtr);
    @Override
    synchronized public ConsolidatedPresence getConsolidatedPresence()  {
        synchronized(core) { 
        
        return ConsolidatedPresence.fromInt(getConsolidatedPresence(nativePtr));
        }
    }

    private native Core getCore(long nativePtr);
    @Override @NonNull
    synchronized public Core getCore()  {
        
        
        return (Core)getCore(nativePtr);
    }

    private native int getIncSubscribePolicy(long nativePtr);
    @Override
    synchronized public SubscribePolicy getIncSubscribePolicy()  {
        synchronized(core) { 
        
        return SubscribePolicy.fromInt(getIncSubscribePolicy(nativePtr));
        }
    }

    private native int setIncSubscribePolicy(long nativePtr, int policy);
    @Override
    synchronized public int setIncSubscribePolicy(SubscribePolicy policy)  {
        synchronized(core) { 
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setIncSubscribePolicy() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        return setIncSubscribePolicy(nativePtr, policy.toInt());
        }
    }

    private native boolean isPresenceReceived(long nativePtr);
    @Override
    synchronized public boolean isPresenceReceived()  {
        synchronized(core) { 
        
        return isPresenceReceived(nativePtr);
        }
    }

    private native String getName(long nativePtr);
    @Override @Nullable
    synchronized public String getName()  {
        synchronized(core) { 
        
        return getName(nativePtr);
        }
    }

    private native int setName(long nativePtr, String name);
    @Override
    synchronized public int setName(@Nullable String name)  {
        synchronized(core) { 
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setName() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        return setName(nativePtr, name);
        }
    }

    private native String[] getPhoneNumbers(long nativePtr);
    @Override @NonNull
    synchronized public String[] getPhoneNumbers()  {
        synchronized(core) { 
        
        return getPhoneNumbers(nativePtr);
        }
    }

    private native PresenceModel getPresenceModel(long nativePtr);
    @Override @Nullable
    synchronized public PresenceModel getPresenceModel()  {
        synchronized(core) { 
        
        return (PresenceModel)getPresenceModel(nativePtr);
        }
    }

    private native void setPresenceModel(long nativePtr, PresenceModel presence);
    @Override
    synchronized public void setPresenceModel(@Nullable PresenceModel presence)  {
        synchronized(core) { 
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setPresenceModel() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setPresenceModel(nativePtr, presence);
        }
    }

    private native String getRefKey(long nativePtr);
    @Override @Nullable
    synchronized public String getRefKey()  {
        synchronized(core) { 
        
        return getRefKey(nativePtr);
        }
    }

    private native void setRefKey(long nativePtr, String key);
    @Override
    synchronized public void setRefKey(@Nullable String key)  {
        synchronized(core) { 
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setRefKey() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setRefKey(nativePtr, key);
        }
    }

    private native boolean subscribesEnabled(long nativePtr);
    @Override
    synchronized public boolean subscribesEnabled()  {
        synchronized(core) { 
        
        return subscribesEnabled(nativePtr);
        }
    }

    private native int enableSubscribes(long nativePtr, boolean enable);
    @Override
    synchronized public int enableSubscribes(boolean enable)  {
        synchronized(core) { 
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call enableSubscribes() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        return enableSubscribes(nativePtr, enable);
        }
    }

    private native int getSubscriptionState(long nativePtr);
    @Override
    synchronized public SubscriptionState getSubscriptionState()  {
        synchronized(core) { 
        
        return SubscriptionState.fromInt(getSubscriptionState(nativePtr));
        }
    }

    private native Vcard getVcard(long nativePtr);
    @Override @Nullable
    synchronized public Vcard getVcard()  {
        synchronized(core) { 
        
        return (Vcard)getVcard(nativePtr);
        }
    }

    private native void setVcard(long nativePtr, Vcard vcard);
    @Override
    synchronized public void setVcard(@Nullable Vcard vcard)  {
        synchronized(core) { 
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setVcard() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setVcard(nativePtr, vcard);
        }
    }

    private native void addAddress(long nativePtr, Address address);
    @Override
    synchronized public void addAddress(@NonNull Address address)  {
        synchronized(core) { 
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call addAddress() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        addAddress(nativePtr, address);
        }
    }

    private native void addPhoneNumber(long nativePtr, String phoneNumber);
    @Override
    synchronized public void addPhoneNumber(@NonNull String phoneNumber)  {
        synchronized(core) { 
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call addPhoneNumber() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        addPhoneNumber(nativePtr, phoneNumber);
        }
    }

    private native boolean createVcard(long nativePtr, String name);
    @Override
    synchronized public boolean createVcard(@Nullable String name)  {
        synchronized(core) { 
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call createVcard() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        return createVcard(nativePtr, name);
        }
    }

    private native void done(long nativePtr);
    @Override
    synchronized public void done()  {
        synchronized(core) { 
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call done() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        done(nativePtr);
        }
    }

    private native void edit(long nativePtr);
    @Override
    synchronized public void edit()  {
        synchronized(core) { 
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call edit() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        edit(nativePtr);
        }
    }

    private native float getCapabilityVersion(long nativePtr, int capability);
    @Override
    synchronized public float getCapabilityVersion(FriendCapability capability)  {
        synchronized(core) { 
        
        return getCapabilityVersion(nativePtr, capability.toInt());
        }
    }

    private native PresenceModel getPresenceModelForUriOrTel(long nativePtr, String uriOrTel);
    @Override @Nullable
    synchronized public PresenceModel getPresenceModelForUriOrTel(@NonNull String uriOrTel)  {
        synchronized(core) { 
        
        return (PresenceModel)getPresenceModelForUriOrTel(nativePtr, uriOrTel);
        }
    }

    private native boolean hasCapability(long nativePtr, int capability);
    @Override
    synchronized public boolean hasCapability(FriendCapability capability)  {
        synchronized(core) { 
        
        return hasCapability(nativePtr, capability.toInt());
        }
    }

    private native boolean hasCapabilityWithVersion(long nativePtr, int capability, float version);
    @Override
    synchronized public boolean hasCapabilityWithVersion(FriendCapability capability, float version)  {
        synchronized(core) { 
        
        return hasCapabilityWithVersion(nativePtr, capability.toInt(), version);
        }
    }

    private native boolean hasCapabilityWithVersionOrMore(long nativePtr, int capability, float version);
    @Override
    synchronized public boolean hasCapabilityWithVersionOrMore(FriendCapability capability, float version)  {
        synchronized(core) { 
        
        return hasCapabilityWithVersionOrMore(nativePtr, capability.toInt(), version);
        }
    }

    private native boolean hasPhoneNumber(long nativePtr, String phoneNumber);
    @Override
    synchronized public boolean hasPhoneNumber(@NonNull String phoneNumber)  {
        synchronized(core) { 
        
        return hasPhoneNumber(nativePtr, phoneNumber);
        }
    }

    private native boolean inList(long nativePtr);
    @Override
    synchronized public boolean inList()  {
        synchronized(core) { 
        
        return inList(nativePtr);
        }
    }

    private native void remove(long nativePtr);
    @Override
    synchronized public void remove()  {
        synchronized(core) { 
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call remove() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        remove(nativePtr);
        }
    }

    private native void removeAddress(long nativePtr, Address address);
    @Override
    synchronized public void removeAddress(@NonNull Address address)  {
        synchronized(core) { 
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call removeAddress() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        removeAddress(nativePtr, address);
        }
    }

    private native void removePhoneNumber(long nativePtr, String phoneNumber);
    @Override
    synchronized public void removePhoneNumber(@NonNull String phoneNumber)  {
        synchronized(core) { 
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call removePhoneNumber() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        removePhoneNumber(nativePtr, phoneNumber);
        }
    }

    private native void save(long nativePtr, Core core);
    @Override
    synchronized public void save(@NonNull Core core)  {
        synchronized(core) { 
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call save() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        save(nativePtr, core);
        }
    }

    private native void setPresenceModelForUriOrTel(long nativePtr, String uriOrTel, PresenceModel presence);
    @Override
    synchronized public void setPresenceModelForUriOrTel(@NonNull String uriOrTel, @Nullable PresenceModel presence)  {
        synchronized(core) { 
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call setPresenceModelForUriOrTel() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        setPresenceModelForUriOrTel(nativePtr, uriOrTel, presence);
        }
    }

    private native Friend newFromVcard(long nativePtr, Vcard vcard);
    @Override @Nullable
    synchronized public Friend newFromVcard(@NonNull Vcard vcard)  {
        synchronized(core) { 
        if (_isConst) {
			try {
				throw new CoreException(this.toString() + " is const! If you want to call newFromVcard() on it, clone it first.");
			} catch (CoreException e) {
				Log.e(e);
				for (StackTraceElement st : e.getStackTrace()) {
					Log.e(st);
				}
			}
		}
        return (Friend)newFromVcard(nativePtr, vcard);
        }
    }

    private native boolean unref(long ptr, boolean isConst);
    protected void finalize() throws Throwable {
        /* Considering the following scenario:
        User scrolls fast in a chat message list that contains a lot of images, 
        so the VM will garbage collect often due to the images taking much space.
        The view itself only has a reference on the EventLogs, not the ChatMessages directly,
        so each call of onBindView will call eventLog.getChatMessage() and the reference will only be temporary.
        It has been observed a race condition in which the JNI level can obtain a LocalRef from a WeakGlobalRef
        of an object that has been scheduled for destruction and for which the finalize() method will be called shortly after.
        The result is a Java object that has been finalized can have it's methods called after the finalize() has terminated.*/
        
        /*The workaround we made here is to keep the nativePtr value even after the unref() only if the object is still reffed by the SDK,
        so the few methods calls following the finalize() will still work because we know the native pointer still exists.*/

        /*This solution will work as long as the objects still have a ref at C level, which is the case with ChatMessages because they are hold by the Events.
        Should we have this problem again in a situation where the C no longer holds a reference (resulting in the C object being destroyed by finalize()), 
        another solution is possible (but more complex to implement).
        This solution would consist in implementing at C level, a queue of belle_sip_object_t that are loosing their last ref. 
        Instead of being deleted immediately by belle_sip_object_unref() calling belle_sip_object_delete(), 
        unref() would place the object onto this "C finalizer queue", the delete() call would then be performed later, 
        by a hook added into the main android looper.
        This solution should work because the actual destruction will be performed from the looper context, 
        so in a place where the local java object causing our troubles no longer exists for sure.
        This solution has however the inconvenience that it can work only within an application that uses a single thread to call liblinphone API.*/
        
        if (nativePtr != 0) {
            boolean destroyed = unref(nativePtr, _isConst);
            if (destroyed) {
                nativePtr = 0;
            }
        }
        super.finalize();
    }

    @Override
    public void setUserData(Object data) {
        userData = data;
    }

    @Override
    public Object getUserData() {
        return userData;
    }

    @Override
    public String toString() {
        return "Java object [" + super.toString() + "], native pointer [" + String.format("0x%08x", nativePtr) + "]";
    }
}
